@@ -1,5 +1,5 @@
 /* Compile a Java program.
-   Copyright (C) 2001-2003, 2006-2023 Free Software Foundation, Inc.
+   Copyright (C) 2001-2003, 2006-2025 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software: you can redistribute it and/or modify
@@ -44,13 +44,13 @@
 #include "concat-filename.h"
 #include "fwriteerror.h"
 #include "clean-temp.h"
-#include "error.h"
+#include <error.h>
 #include "xvasprintf.h"
 #include "verify.h"
 #include "c-strstr.h"
 #include "gettext.h"
 
-#define _(str) gettext (str)
+#define _(msgid) dgettext ("gnulib", msgid)
 
 
 /* Survey of Java compilers.
@@ -243,7 +243,7 @@ compile_using_envjavac (const char *javac,
   argv[1] = "-c";
   argv[2] = command;
   argv[3] = NULL;
-  exitstatus = execute (javac, BOURNE_SHELL, argv, NULL,
+  exitstatus = execute (javac, BOURNE_SHELL, argv, NULL, NULL,
                         false, false, false, null_stderr,
                         true, true, NULL);
   err = (exitstatus != 0);
@@ -315,7 +315,7 @@ compile_using_javac (const char * const *java_sources,
       free (command);
     }
 
-  exitstatus = execute ("javac", "javac", argv, NULL,
+  exitstatus = execute ("javac", "javac", argv, NULL, NULL,
                         false, false, false,
                         null_stderr, true, true, NULL);
   err = (exitstatus != 0);
@@ -343,7 +343,7 @@ execute_and_read_line (const char *progname,
   int exitstatus;
 
   /* Open a pipe to the program.  */
-  child = create_pipe_in (progname, prog_path, prog_argv, NULL,
+  child = create_pipe_in (progname, prog_path, prog_argv, NULL, NULL,
                           DEV_NULL, false, true, false, fd);
 
   if (child == -1)
@@ -352,16 +352,15 @@ execute_and_read_line (const char *progname,
   /* Retrieve its result.  */
   fp = fdopen (fd[0], "r");
   if (fp == NULL)
-    {
-      error (0, errno, _("fdopen() failed"));
-      return NULL;
-    }
+    error (EXIT_FAILURE, errno, _("fdopen() failed"));
 
   line = NULL; linesize = 0;
   linelen = getline (&line, &linesize, fp);
   if (linelen == (size_t)(-1))
     {
       error (0, 0, _("%s subprocess I/O error"), progname);
+      fclose (fp);
+      wait_subprocess (child, progname, true, false, true, false, NULL);
       return NULL;
     }
   if (linelen > 0 && line[linelen - 1] == '\n')
@@ -765,7 +764,7 @@ is_javac_present (void)
 
       argv[0] = "javac";
       argv[1] = NULL;
-      exitstatus = execute ("javac", "javac", argv, NULL,
+      exitstatus = execute ("javac", "javac", argv, NULL, NULL,
                             false, false, true, true,
                             true, false, NULL);
       javac_present = (exitstatus == 0 || exitstatus == 1 || exitstatus == 2);
