@@ -91,8 +91,7 @@ main ()
   {
     sf_istream_t stream;
     sf_istream_init_from_string_desc (&stream,
-                                      sd_new_addr (CONTENTS_LEN,
-                                                   (char *) contents));
+                                      sd_new_addr (CONTENTS_LEN, contents));
     test_open_stream (&stream);
     sf_free (&stream);
   }
